@@ -3,30 +3,32 @@ package com.unciv.json
 import com.badlogic.gdx.Gdx
 import com.badlogic.gdx.files.FileHandle
 import com.badlogic.gdx.utils.Json
+import com.badlogic.gdx.utils.JsonWriter
 import com.badlogic.gdx.utils.SerializationException
+import com.unciv.logic.civilization.CivRankingHistory
 import com.unciv.logic.map.tile.TileHistory
-import com.unciv.ui.components.KeyCharAndCode
-import com.unciv.ui.components.KeyboardBindings
+import com.unciv.ui.components.input.KeyCharAndCode
+import com.unciv.ui.components.input.KeyboardBindings
 import java.time.Duration
 
 
 /**
  * [Json] is not thread-safe. Use a new one for each parse.
  */
-fun json() = Json().apply {
+fun json() = Json(JsonWriter.OutputType.json).apply {
+    // Gdx default output type is JsonWriter.OutputType.minimal, which generates invalid Json - e.g. most quotes removed.
+    // The constructor parameter above changes that to valid Json
+    // Note an instance set to json can read minimal and vice versa
+
     setIgnoreDeprecated(true)
     ignoreUnknownFields = true
 
-    // Default output type is JsonWriter.OutputType.minimal, which generates invalid Json - e.g. most quotes removed.
-    // To get better Json, use:
-    // setOutputType(JsonWriter.OutputType.json)
-    // Note an instance set to json can read minimal and vice versa
-
     setSerializer(HashMapVector2.getSerializerClass(), HashMapVector2.createSerializer())
     setSerializer(Duration::class.java, DurationSerializer())
     setSerializer(KeyCharAndCode::class.java, KeyCharAndCode.Serializer())
     setSerializer(KeyboardBindings::class.java, KeyboardBindings.Serializer())
     setSerializer(TileHistory::class.java, TileHistory.Serializer())
+    setSerializer(CivRankingHistory::class.java, CivRankingHistory.Serializer())
 }
 
 /**
