@@ -108,8 +108,8 @@ class WorkerAutomation(
 
     companion object {
         /** Maps to instance [WorkerAutomation.automateWorkerAction] knowing only the MapUnit */
-        fun automateWorkerAction(unit: MapUnit) {
-            unit.civ.getWorkerAutomation().automateWorkerAction(unit)
+        fun automateWorkerAction(unit: MapUnit, tilesWhereWeWillBeCaptured: Set<Tile>) {
+            unit.civ.getWorkerAutomation().automateWorkerAction(unit, tilesWhereWeWillBeCaptured)
         }
 
         /** Convenience shortcut supports old calling syntax for [WorkerAutomation.getPriority] */
@@ -131,9 +131,9 @@ class WorkerAutomation(
     /**
      * Automate one Worker - decide what to do and where, move, start or continue work.
      */
-    fun automateWorkerAction(unit: MapUnit) {
+    fun automateWorkerAction(unit: MapUnit, tilesWhereWeWillBeCaptured: Set<Tile>) {
         val currentTile = unit.getTile()
-        val tileToWork = findTileToWork(unit)
+        val tileToWork = findTileToWork(unit, tilesWhereWeWillBeCaptured)
 
         if (getPriority(tileToWork, civInfo) < 3) { // building roads is more important
             if (tryConnectingCities(unit)) return
@@ -201,7 +201,7 @@ class WorkerAutomation(
 
         // Idle CS units should wander so they don't obstruct players so much
         if (unit.civ.isCityState())
-            wander(unit, stayInTerritory = true)
+            wander(unit, stayInTerritory = true, tilesToAvoid = tilesWhereWeWillBeCaptured)
     }
 
     /**
@@ -221,13 +221,15 @@ class WorkerAutomation(
         if (candidateCities.none()) return false // do nothing.
 
         val isCandidateTilePredicate: (Tile) -> Boolean = { it.isLand && unit.movement.canPassThrough(it) }
+        val getTilePriority: (Tile) -> Int = { it.roadStatus.ordinal }
         val currentTile = unit.getTile()
         val cityTilesToSeek = ArrayList(tilesOfConnectedCities.sortedBy { it.aerialDistanceTo(currentTile) })
 
         for (toConnectCity in candidateCities) {
             val toConnectTile = toConnectCity.getCenterTile()
+
             val bfs: BFS = bfsCache[toConnectTile.position] ?:
-                BFS(toConnectTile, isCandidateTilePredicate).apply {
+                BFS(toConnectTile, getTilePriority, isCandidateTilePredicate).apply {
                     maxSize = HexMath.getNumberOfTilesInHexagon(
                         WorkerAutomationConst.maxBfsReachPadding +
                             tilesOfConnectedCities.minOf { it.aerialDistanceTo(toConnectTile) }
@@ -238,6 +240,7 @@ class WorkerAutomation(
             while (true) {
                 for (cityTile in cityTilesToSeek.toList()) { // copy since we change while running
                     if (!bfs.hasReachedTile(cityTile)) continue
+
                     // we have a winner!
                     val pathToCity = bfs.getPathTo(cityTile)
                     val roadableTiles = pathToCity.filter { it.getUnpillagedRoad() < bestRoadAvailable }
@@ -279,27 +282,29 @@ class WorkerAutomation(
      * Looks for a worthwhile tile to improve
      * @return The current tile if no tile to work was found
      */
-    private fun findTileToWork(unit: MapUnit): Tile {
+    private fun findTileToWork(unit: MapUnit, tilesToAvoid: Set<Tile>): Tile {
         val currentTile = unit.getTile()
         val workableTiles = currentTile.getTilesInDistance(4)
                 .filter {
-                    (it.civilianUnit == null || it == currentTile)
-                            && (it.owningCity == null || it.getOwner()==civInfo)
-                            && getPriority(it) > 1
-                            && it.getTilesInDistance(2)  // don't work in range of enemy cities
+                    it !in tilesToAvoid
+                    && (it.civilianUnit == null || it == currentTile)
+                    && (it.owningCity == null || it.getOwner()==civInfo)
+                    && getPriority(it) > 1
+                    && it.getTilesInDistance(2)  // don't work in range of enemy cities
                         .none { tile -> tile.isCityCenter() && tile.getCity()!!.civ.isAtWarWith(civInfo) }
-                            && it.getTilesInDistance(3)  // don't work in range of enemy units
+                    && it.getTilesInDistance(3)  // don't work in range of enemy units
                         .none { tile -> tile.militaryUnit != null && tile.militaryUnit!!.civ.isAtWarWith(civInfo)}
                 }
                 .sortedByDescending { getPriority(it) }
 
         // These are the expensive calculations (tileCanBeImproved, canReach), so we only apply these filters after everything else it done.
-        val selectedTile = workableTiles.firstOrNull { unit.movement.canReach(it) && (tileCanBeImproved(unit, it) || it.isPillaged()) }
+        val selectedTile =
+            workableTiles.firstOrNull { unit.movement.canReach(it) && (tileCanBeImproved(unit, it) || it.isPillaged()) }
+                ?: return currentTile
 
-        return if (selectedTile != null
-                && ((!tileCanBeImproved(unit, currentTile) && !currentTile.isPillaged()) // current tile is unimprovable
+        return if ((!tileCanBeImproved(unit, currentTile) && !currentTile.isPillaged()) // current tile is unimprovable
                     || !workableTiles.contains(currentTile) // current tile is unworkable by city
-                    || getPriority(selectedTile) > getPriority(currentTile)))  // current tile is less important
+                    || getPriority(selectedTile) > getPriority(currentTile))  // current tile is less important
             selectedTile
         else currentTile
     }
@@ -311,9 +316,11 @@ class WorkerAutomation(
     private fun tileCanBeImproved(unit: MapUnit, tile: Tile): Boolean {
         if (!tile.isLand || tile.isImpassible() || tile.isCityCenter())
             return false
+
         val city = tile.getCity()
         if (city == null || city.civ != civInfo)
             return false
+
         if (!city.tilesInRange.contains(tile)
                 && !tile.hasViewableResource(civInfo)
                 && civInfo.cities.none { it.getCenterTile().aerialDistanceTo(tile) <= 3 })
@@ -325,17 +332,16 @@ class WorkerAutomation(
                 && unit.civ.isHuman())
             return false
 
-
-
         if (tile.improvement == null || junkImprovement == true) {
             if (tile.improvementInProgress != null && unit.canBuildImprovement(tile.getTileImprovementInProgress()!!, tile)) return true
             val chosenImprovement = chooseImprovement(unit, tile)
             if (chosenImprovement != null && tile.improvementFunctions.canBuildImprovement(chosenImprovement, civInfo) && unit.canBuildImprovement(chosenImprovement, tile)) return true
         } else if (!tile.containsGreatImprovement() && tile.hasViewableResource(civInfo)
-            && tile.tileResource.isImprovedBy(tile.improvement!!)
+            && !tile.tileResource.isImprovedBy(tile.improvement!!)
             && (chooseImprovement(unit, tile) // if the chosen improvement is not null and buildable
                 .let { it != null && tile.improvementFunctions.canBuildImprovement(it, civInfo) && unit.canBuildImprovement(it, tile)}))
             return true
+
         return false // couldn't find anything to construct here
     }
 
@@ -387,16 +393,15 @@ class WorkerAutomation(
 
         val lastTerrain = tile.lastTerrain
 
-        fun isUnbuildableAndRemovable(terrain: Terrain): Boolean = terrain.unbuildable
-                && ruleSet.tileImprovements.containsKey(Constants.remove + terrain.name)
-
+        fun isRemovable(terrain: Terrain): Boolean = ruleSet.tileImprovements.containsKey(Constants.remove + terrain.name)
 
         val improvementStringForResource: String? = when {
             tile.resource == null || !tile.hasViewableResource(civInfo) -> null
             tile.terrainFeatures.isNotEmpty()
-                    && isUnbuildableAndRemovable(lastTerrain)
-                    && !tile.providesResources(civInfo)
-                    && !isResourceImprovementAllowedOnFeature(tile, potentialTileImprovements) -> Constants.remove + lastTerrain.name
+                && lastTerrain.unbuildable
+                && isRemovable(lastTerrain)
+                && !tile.providesResources(civInfo)
+                && !isResourceImprovementAllowedOnFeature(tile, potentialTileImprovements) -> Constants.remove + lastTerrain.name
             else -> tile.tileResource.getImprovements().filter { it in potentialTileImprovements || it==tile.improvement }
                 .maxByOrNull { getRankingWithImprovement(it) }
         }
@@ -409,12 +414,14 @@ class WorkerAutomation(
             tile.resource != null && tile.tileResource.getImprovements().any() -> return null
             bestBuildableImprovement == null -> null
 
-            tile.improvement!=null && getRankingWithImprovement(tile.improvement!!) > getRankingWithImprovement(bestBuildableImprovement.name)
+            tile.improvement != null &&
+                    getRankingWithImprovement(tile.improvement!!) > getRankingWithImprovement(bestBuildableImprovement.name)
                 -> null // What we have is better, even if it's pillaged we should repair it
 
             lastTerrain.let {
-                isUnbuildableAndRemovable(it) &&
-                        (Automation.rankStatsValue(it, civInfo) < 0 || it.hasUnique(UniqueType.NullifyYields) )
+                isRemovable(it) &&
+                    (Automation.rankStatsValue(it, civInfo) < 0
+                        || it.hasUnique(UniqueType.NullifyYields))
             } -> Constants.remove + lastTerrain.name
 
             else -> bestBuildableImprovement.name
