@@ -57,8 +57,9 @@ class CityExpansionManager : IsPartOfGameInfoSerialization {
 
     fun canBuyTile(tile: Tile): Boolean {
         return when {
-            city.isPuppet -> false
+            city.isPuppet || city.isBeingRazed -> false
             tile.getOwner() != null -> false
+            city.isInResistance() -> false
             tile !in city.tilesInRange -> false
             else -> tile.neighbors.any { it.getCity() == city }
         }
@@ -72,6 +73,9 @@ class CityExpansionManager : IsPartOfGameInfoSerialization {
             throw NotEnoughGoldToBuyTileException()
         city.civ.addGold(-goldCost)
         takeOwnership(tile)
+
+        // Reapply worked tiles optimization (aka CityFocus) - doing it here means AI profits too
+        city.reassignPopulationDeferred()
     }
 
     fun getGoldCostOfTile(tile: Tile): Int {
@@ -147,7 +151,7 @@ class CityExpansionManager : IsPartOfGameInfoSerialization {
 
         tile.setOwningCity(null)
 
-        city.civ.cache.updateCivResources()
+        city.civ.cache.updateOurTiles()
         city.cityStats.update()
 
         tile.history.recordRelinquishOwnership(tile)
@@ -169,14 +173,13 @@ class CityExpansionManager : IsPartOfGameInfoSerialization {
         city.tiles = city.tiles.withItem(tile.position)
         tile.setOwningCity(city)
         city.population.autoAssignPopulation()
-        city.civ.cache.updateCivResources()
+        city.civ.cache.updateOurTiles()
         city.cityStats.update()
 
         for (unit in tile.getUnits().toList()) // toListed because we're modifying
             if (!unit.civ.diplomacyFunctions.canPassThroughTiles(city.civ))
                 unit.movement.teleportToClosestMoveableTile()
 
-        city.civ.cache.updateViewableTiles()
 
         tile.history.recordTakeOwnership(tile)
     }
@@ -187,7 +190,8 @@ class CityExpansionManager : IsPartOfGameInfoSerialization {
             val location = addNewTileWithCulture()
             if (location != null) {
                 val locations = LocationAction(location, city.location)
-                city.civ.addNotification("[" + city.name + "] has expanded its borders!", locations, NotificationCategory.Cities, NotificationIcon.Culture)
+                city.civ.addNotification("[${city.name}] has expanded its borders!", locations,
+                    NotificationCategory.Cities, NotificationIcon.Culture)
             }
         }
     }
