@@ -138,8 +138,31 @@ class CivInfoTransientCache(val civInfo: Civilization) {
         civInfo.viewableInvisibleUnitsTiles = newViewableInvisibleTiles
     }
 
+    var ourTilesAndNeighboringTiles: Set<Tile> = HashSet()
+
+    /** Our tiles update pretty infrequently - most 'viewable tile' changes are due to unit movements,
+     * which means we can store this separately and use it 'as is' so we don't need to find the neighboring tiles every time
+     * a unit moves */
+    fun updateOurTiles() {
+        ourTilesAndNeighboringTiles = civInfo.cities.asSequence()
+            .flatMap { it.getTiles() } // our owned tiles, still distinct
+            .flatMap { sequenceOf(it) + it.neighbors }
+            // now we got a mix of owned, unowned and competitor-owned tiles, and **duplicates**
+            // but Sequence.toSet is just as good at making them distinct as any other operation
+            .toSet()
+
+        updateViewableTiles()
+        updateCivResources()
+    }
+
     private fun setNewViewableTiles() {
-        val newViewableTiles = HashSet<Tile>()
+        if (civInfo.isDefeated()) {
+            // Avoid meeting dead city states when entering a tile owned by their former ally (#9245)
+            // In that case ourTilesAndNeighboringTiles and getCivUnits will be empty, but the for
+            // loop getKnownCivs/getAllyCiv would add tiles.
+            civInfo.viewableTiles = emptySet()
+            return
+        }
 
         // while spectating all map is visible
         if (civInfo.isSpectator() || DebugUtils.VISIBLE_MAP) {
@@ -149,13 +172,7 @@ class CivInfoTransientCache(val civInfo: Civilization) {
             return
         }
 
-        // There are a LOT of tiles usually.
-        // And making large lists of them just as intermediaries before we shove them into the hashset is very space-inefficient.
-        // And so, sequences to the rescue!
-        val ownedTiles = civInfo.cities.asSequence().flatMap { it.getTiles() }
-        newViewableTiles.addAll(ownedTiles)
-        val neighboringUnownedTiles = ownedTiles.flatMap { tile -> tile.neighbors.filter { it.getOwner() != civInfo } }
-        newViewableTiles.addAll(neighboringUnownedTiles)
+        val newViewableTiles = HashSet<Tile>(ourTilesAndNeighboringTiles)
         newViewableTiles.addAll(civInfo.units.getCivUnits().flatMap { unit -> unit.viewableTiles.asSequence().filter { it.getOwner() != civInfo } })
 
         for (otherCiv in civInfo.getKnownCivs()) {
@@ -196,9 +213,10 @@ class CivInfoTransientCache(val civInfo: Civilization) {
             if (civInfo.naturalWonders.contains(tile.naturalWonder))
                 continue
             civInfo.naturalWonders.add(tile.naturalWonder!!)
-            if(!civInfo.isSpectator())
-                civInfo.addNotification("We have discovered [${tile.naturalWonder}]!",
-                    tile.position, NotificationCategory.General, "StatIcons/Happiness")
+            if (civInfo.isSpectator()) continue // don't trigger anything
+
+            civInfo.addNotification("We have discovered [${tile.naturalWonder}]!",
+                tile.position, NotificationCategory.General, "StatIcons/Happiness")
 
             var goldGained = 0
             val discoveredNaturalWonders = civInfo.gameInfo.civilizations.filter { it != civInfo && it.isMajorCiv() }
@@ -223,8 +241,6 @@ class CivInfoTransientCache(val civInfo: Civilization) {
                 StateForConditionals(civInfo, tile = tile)
             ))
                 UniqueTriggerActivation.triggerCivwideUnique(unique, civInfo, tile=tile, triggerNotificationText = "due to discovering a Natural Wonder")
-
-
         }
     }
 
@@ -237,24 +253,33 @@ class CivInfoTransientCache(val civInfo: Civilization) {
     fun updateCitiesConnectedToCapital(initialSetup: Boolean = false) {
         if (civInfo.cities.isEmpty() || civInfo.getCapital() == null) return // eg barbarians
 
-        val citiesReachedToMediums = CapitalConnectionsFinder(civInfo).find()
+        val oldConnectedCities = if (initialSetup)
+            civInfo.cities.filter { it.connectedToCapitalStatus == City.ConnectedToCapitalStatus.`true` }
+            else citiesConnectedToCapitalToMediums.keys
+        val oldMaybeConnectedCities = if (initialSetup)
+            civInfo.cities.filter { it.connectedToCapitalStatus != City.ConnectedToCapitalStatus.`false` }
+        else citiesConnectedToCapitalToMediums.keys
 
-        if (!initialSetup) { // In the initial setup we're loading an old game state, so it doesn't really count
-            for (city in citiesReachedToMediums.keys)
-                if (city !in citiesConnectedToCapitalToMediums && city.civ == civInfo && city != civInfo.getCapital()!!)
-                    civInfo.addNotification("[${city.name}] has been connected to your capital!",
-                        city.location, NotificationCategory.Cities, NotificationIcon.Gold
-                    )
+        citiesConnectedToCapitalToMediums = CapitalConnectionsFinder(civInfo).find()
 
-            // This may still contain cities that have just been destroyed by razing - thus the population test
-            for (city in citiesConnectedToCapitalToMediums.keys)
-                if (!citiesReachedToMediums.containsKey(city) && city.civ == civInfo && city.population.population > 0)
-                    civInfo.addNotification("[${city.name}] has been disconnected from your capital!",
-                        city.location, NotificationCategory.Cities, NotificationIcon.Gold
-                    )
-        }
+        val newConnectedCities = citiesConnectedToCapitalToMediums.keys
 
-        citiesConnectedToCapitalToMediums = citiesReachedToMediums
+        for (city in newConnectedCities)
+            if (city !in oldMaybeConnectedCities && city.civ == civInfo && city != civInfo.getCapital()!!)
+                civInfo.addNotification("[${city.name}] has been connected to your capital!",
+                    city.location, NotificationCategory.Cities, NotificationIcon.Gold
+                )
+
+        // This may still contain cities that have just been destroyed by razing - thus the population test
+        for (city in oldConnectedCities)
+            if (city !in newConnectedCities && city.civ == civInfo && city.population.population > 0)
+                civInfo.addNotification("[${city.name}] has been disconnected from your capital!",
+                    city.location, NotificationCategory.Cities, NotificationIcon.Gold
+                )
+
+        for (city in civInfo.cities)
+            city.connectedToCapitalStatus = if (city in newConnectedCities)
+                City.ConnectedToCapitalStatus.`true` else City.ConnectedToCapitalStatus.`false`
     }
 
     fun updateCivResources() {
@@ -262,7 +287,7 @@ class CivInfoTransientCache(val civInfo: Civilization) {
         for (city in civInfo.cities) newDetailedCivResources.add(city.getCityResources())
 
         for (resourceSupply in newDetailedCivResources)
-            if(resourceSupply.amount > 0)
+            if (resourceSupply.amount > 0)
                 resourceSupply.amount = (resourceSupply.amount * civInfo.getResourceModifier(resourceSupply.resource)).toInt()
 
         if (!civInfo.isCityState()) {
@@ -295,13 +320,13 @@ class CivInfoTransientCache(val civInfo: Civilization) {
 
         for (unit in civInfo.units.getCivUnits())
             newDetailedCivResources.subtractResourceRequirements(
-                unit.baseUnit.getResourceRequirements(), civInfo.gameInfo.ruleset, "Units")
+                unit.baseUnit.getResourceRequirementsPerTurn(), civInfo.gameInfo.ruleset, "Units")
 
         // Check if anything has actually changed so we don't update stats for no reason - this uses List equality which means it checks the elements
         if (civInfo.detailedCivResources == newDetailedCivResources) return
 
         civInfo.detailedCivResources = newDetailedCivResources
-        civInfo.summarizedCivResources = newDetailedCivResources.sumByResource("All")
+        civInfo.summarizedCivResourceSupply = newDetailedCivResources.sumByResource("All")
 
         civInfo.updateStatsForNextTurn() // More or less resources = more or less happiness, with potential domino effects
     }
