@@ -109,8 +109,10 @@ class Unique(val text: String, val sourceObjectType: UniqueTarget? = null, val s
         // filter out possible replacements that are obviously wrong
         val uniquesWithNoErrors = finalPossibleUniques.filter {
             val unique = Unique(it)
-            val errors = RulesetValidator(ruleset).checkUnique(unique, true, "",
-                UniqueType.UniqueComplianceErrorSeverity.RulesetSpecific, unique.type!!.targetTypes.first())
+            val errors = RulesetValidator(ruleset).checkUnique(
+                unique, true, "",
+                UniqueType.UniqueComplianceErrorSeverity.RulesetSpecific
+            )
             errors.isEmpty()
         }
         if (uniquesWithNoErrors.size == 1) return uniquesWithNoErrors.first()
@@ -125,7 +127,7 @@ class Unique(val text: String, val sourceObjectType: UniqueTarget? = null, val s
     ): Boolean {
 
         val nonConditionalConditionTypes = setOf(UniqueTarget.TriggerCondition, UniqueTarget.UnitTriggerCondition, UniqueTarget.UnitActionModifier)
-        if (condition.type!!.targetTypes.any { it in nonConditionalConditionTypes })
+        if (condition.type?.targetTypes?.any { it in nonConditionalConditionTypes } == true)
             return true // not a filtering condition
 
         fun ruleset() = state.civInfo!!.gameInfo.ruleset
@@ -174,21 +176,26 @@ class Unique(val text: String, val sourceObjectType: UniqueTarget? = null, val s
             UniqueType.ConditionalWLTKD ->
                 state.city != null && state.city.isWeLoveTheKingDayActive()
             UniqueType.ConditionalBeforeEra ->
-                state.civInfo != null && state.civInfo.getEraNumber() < ruleset().eras[condition.params[0]]!!.eraNumber
+                state.civInfo != null && ruleset().eras.containsKey(condition.params[0])
+                    && state.civInfo.getEraNumber() < ruleset().eras[condition.params[0]]!!.eraNumber
             UniqueType.ConditionalStartingFromEra ->
-                state.civInfo != null && state.civInfo.getEraNumber() >= ruleset().eras[condition.params[0]]!!.eraNumber
+                state.civInfo != null && ruleset().eras.containsKey(condition.params[0])
+                    &&  state.civInfo.getEraNumber() >= ruleset().eras[condition.params[0]]!!.eraNumber
             UniqueType.ConditionalDuringEra ->
-                state.civInfo != null && state.civInfo.getEraNumber() == ruleset().eras[condition.params[0]]!!.eraNumber
+                state.civInfo != null && ruleset().eras.containsKey(condition.params[0])
+                    &&  state.civInfo.getEraNumber() == ruleset().eras[condition.params[0]]!!.eraNumber
             UniqueType.ConditionalIfStartingInEra ->
                 state.civInfo != null && state.civInfo.gameInfo.gameParameters.startingEra == condition.params[0]
             UniqueType.ConditionalTech ->
                 state.civInfo != null && state.civInfo.tech.isResearched(condition.params[0])
             UniqueType.ConditionalNoTech ->
                 state.civInfo != null && !state.civInfo.tech.isResearched(condition.params[0])
-            UniqueType.ConditionalAfterPolicy ->
-                state.civInfo != null && state.civInfo.policies.isAdopted(condition.params[0])
-            UniqueType.ConditionalBeforePolicy ->
+            UniqueType.ConditionalAfterPolicyOrBelief ->
+                state.civInfo != null && (state.civInfo.policies.isAdopted(condition.params[0])
+                    || state.civInfo.religionManager.religion?.hasBelief(condition.params[0]) == true)
+            UniqueType.ConditionalBeforePolicyOrBelief ->
                 state.civInfo != null && !state.civInfo.policies.isAdopted(condition.params[0])
+                    && state.civInfo.religionManager.religion?.hasBelief(condition.params[0]) != true
             UniqueType.ConditionalBeforePantheon ->
                 state.civInfo != null && state.civInfo.religionManager.religionState == ReligionState.None
             UniqueType.ConditionalAfterPantheon ->
