@@ -10,21 +10,19 @@ import com.badlogic.gdx.scenes.scene2d.actions.Actions
 import com.badlogic.gdx.scenes.scene2d.ui.Table
 import com.badlogic.gdx.utils.Align
 import com.unciv.GUI
-import com.unciv.UncivGame
 import com.unciv.logic.battle.CityCombatant
 import com.unciv.logic.city.City
-import com.unciv.logic.city.INonPerpetualConstruction
-import com.unciv.logic.city.PerpetualConstruction
+import com.unciv.models.ruleset.INonPerpetualConstruction
+import com.unciv.models.ruleset.PerpetualConstruction
 import com.unciv.logic.civilization.diplomacy.RelationshipLevel
 import com.unciv.models.TutorialTrigger
-import com.unciv.models.translations.tr
 import com.unciv.ui.components.BorderedTable
 import com.unciv.ui.components.Fonts
 import com.unciv.ui.components.extensions.center
 import com.unciv.ui.components.extensions.centerX
 import com.unciv.ui.components.extensions.colorFromRGB
 import com.unciv.ui.components.extensions.darken
-import com.unciv.ui.components.extensions.onClick
+import com.unciv.ui.components.input.onClick
 import com.unciv.ui.components.extensions.toGroup
 import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.images.ImageGetter
@@ -33,6 +31,7 @@ import com.unciv.ui.screens.basescreen.BaseScreen
 import com.unciv.ui.screens.cityscreen.CityReligionInfoTable
 import com.unciv.ui.screens.cityscreen.CityScreen
 import com.unciv.ui.screens.diplomacyscreen.DiplomacyScreen
+import com.unciv.utils.DebugUtils
 import kotlin.math.max
 import kotlin.math.min
 
@@ -228,7 +227,7 @@ private class CityTable(city: City, forPopup: Boolean = false) : BorderedTable(
         pad(0f)
         defaults().pad(0f)
 
-        val isShowDetailedInfo = UncivGame.Current.viewEntireMapForDebug
+        val isShowDetailedInfo = DebugUtils.VISIBLE_MAP
                 || city.civ == viewingCiv
                 || viewingCiv.isSpectator()
 
@@ -312,7 +311,7 @@ private class CityTable(city: City, forPopup: Boolean = false) : BorderedTable(
             table.add(capitalIcon).size(20f).padRight(5f)
         }
 
-        val cityName = city.name.tr().toLabel(fontColor = textColor, alignment = Align.center)
+        val cityName = city.name.toLabel(fontColor = textColor, alignment = Align.center, hideIcons = true)
         table.add(cityName).growY().center()
 
         if (!forPopup) {
@@ -532,7 +531,7 @@ class CityButton(val city: City, private val tileGroup: TileGroup): Table(BaseSc
             if (isButtonMoved) {
                 // second tap on the button will go to the city screen
                 // if this city belongs to you and you are not iterating though the air units
-                if (GUI.isDebugMapVisible() || viewingPlayer.isSpectator()
+                if (DebugUtils.VISIBLE_MAP || viewingPlayer.isSpectator()
                     || (belongsToViewingCiv() && !tileGroup.tile.airUnits.contains(unitTable.selectedUnit))) {
                         GUI.pushScreen(CityScreen(city))
                 } else if (viewingPlayer.knows(city.civ)) {
@@ -594,7 +593,9 @@ class CityButton(val city: City, private val tileGroup: TileGroup): Table(BaseSc
             add(CityReligionInfoTable(city.religion, true)).colspan(3).row()
             addOKButton("Diplomacy") { openDiplomacy() }
             add().expandX()
-            addCloseButton()
+            addCloseButton() {
+                GUI.getWorldScreen().run { nextTurnButton.update(this@run) }
+            }
         }
         popup.open()
     }
