@@ -9,14 +9,14 @@ import com.unciv.ui.components.ExpanderTab
 import com.unciv.ui.components.extensions.addBorder
 import com.unciv.ui.components.extensions.addSeparatorVertical
 import com.unciv.ui.components.extensions.darken
-import com.unciv.ui.components.extensions.onClick
+import com.unciv.ui.components.input.onClick
 import com.unciv.ui.components.extensions.surroundWithCircle
 import com.unciv.ui.components.extensions.toGroup
 import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.screens.basescreen.BaseScreen
 
-class SpecialistAllocationTable(val cityScreen: CityScreen) : Table(BaseScreen.skin) {
+class SpecialistAllocationTable(private val cityScreen: CityScreen) : Table(BaseScreen.skin) {
     val cityInfo = cityScreen.city
 
     fun update() {
@@ -27,7 +27,7 @@ class SpecialistAllocationTable(val cityScreen: CityScreen) : Table(BaseScreen.s
         if (cityScreen.canCityBeChanged()) {
             if (cityInfo.manualSpecialists) {
                 val manualSpecialists = "Manual Specialists".toLabel()
-                    .addBorder(5f, BaseScreen.skin.get("color", Color::class.java))
+                    .addBorder(5f, BaseScreen.skin.getColor("color"))
                 manualSpecialists.onClick {
                     cityInfo.manualSpecialists = false
                     cityInfo.reassignPopulation(); cityScreen.update()
@@ -35,7 +35,7 @@ class SpecialistAllocationTable(val cityScreen: CityScreen) : Table(BaseScreen.s
                 add(manualSpecialists).colspan(5).row()
             } else {
                 val autoSpecialists = "Auto Specialists".toLabel()
-                    .addBorder(5f, BaseScreen.skin.get("color", Color::class.java))
+                    .addBorder(5f, BaseScreen.skin.getColor("color"))
                 autoSpecialists.onClick { cityInfo.manualSpecialists = true; update() }
                 add(autoSpecialists).colspan(5).row()
             }
@@ -44,7 +44,7 @@ class SpecialistAllocationTable(val cityScreen: CityScreen) : Table(BaseScreen.s
             if (!cityInfo.getRuleset().specialists.containsKey(specialistName)) // specialist doesn't exist in this ruleset, probably a mod
                 continue
             val newSpecialists = cityInfo.population.getNewSpecialists()
-            val assignedSpecialists = newSpecialists[specialistName]!!
+            val assignedSpecialists = newSpecialists[specialistName]
 
             if (cityScreen.canChangeState) add(getUnassignButton(assignedSpecialists, specialistName))
             add(getAllocationTable(assignedSpecialists, maxSpecialists, specialistName)).pad(10f)
@@ -56,7 +56,7 @@ class SpecialistAllocationTable(val cityScreen: CityScreen) : Table(BaseScreen.s
     }
 
 
-    fun getAllocationTable(assignedSpecialists: Int, maxSpecialists: Int, specialistName: String): Table {
+    private fun getAllocationTable(assignedSpecialists: Int, maxSpecialists: Int, specialistName: String): Table {
 
         val specialistIconTable = Table()
         val specialistObject = cityInfo.getRuleset().specialists[specialistName]!!
@@ -65,6 +65,7 @@ class SpecialistAllocationTable(val cityScreen: CityScreen) : Table(BaseScreen.s
             else Color.GRAY // unassigned
             val icon = ImageGetter.getSpecialistIcon(color)
             specialistIconTable.add(icon).size(30f)
+            if (i % 5 == 0) specialistIconTable.row()
         }
         return specialistIconTable
     }
@@ -106,10 +107,20 @@ class SpecialistAllocationTable(val cityScreen: CityScreen) : Table(BaseScreen.s
     private fun getSpecialistStatsTable(specialistName: String): Table {
         val specialistStatTable = Table().apply { defaults().padBottom(5f).padTop(5f) }
         val specialistStats = cityInfo.cityStats.getStatsOfSpecialist(specialistName)
+        var itemsInRow = 0
+        fun incrementItemsInRow(){
+            itemsInRow++
+            if (itemsInRow % 3 == 0){
+                itemsInRow = 0
+                specialistStatTable.row()
+            }
+        }
+
         for ((key, value) in specialistStats) {
             if (value == 0f) continue
             specialistStatTable.add(value.toInt().toLabel())
             specialistStatTable.add(ImageGetter.getStatIcon(key.name)).size(20f).padRight(10f)
+            incrementItemsInRow()
         }
 
         val specialist = cityInfo.getRuleset().specialists[specialistName]!!
@@ -117,6 +128,7 @@ class SpecialistAllocationTable(val cityScreen: CityScreen) : Table(BaseScreen.s
         for (s in specialist.greatPersonPoints) {
             specialistStatTable.add(s.value.toLabel())
             specialistStatTable.add(ImageGetter.getUnitIcon(s.key, Color.GOLD).toGroup(20f)).padRight(10f)
+            incrementItemsInRow()
         }
 
         return specialistStatTable
