@@ -3,18 +3,15 @@ package com.unciv.ui.screens.overviewscreen
 import com.badlogic.gdx.graphics.Color
 import com.badlogic.gdx.scenes.scene2d.Touchable
 import com.badlogic.gdx.scenes.scene2d.ui.Table
-import com.unciv.GUI
-import com.unciv.UncivGame
 import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.civilization.Notification
 import com.unciv.logic.civilization.NotificationCategory
+import com.unciv.ui.components.ColorMarkupLabel
+import com.unciv.ui.components.TabbedPager
+import com.unciv.ui.components.input.onClick
+import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.screens.basescreen.BaseScreen
-import com.unciv.ui.components.TabbedPager
-import com.unciv.ui.components.WrappableLabel
-import com.unciv.ui.components.extensions.onClick
-import com.unciv.ui.components.extensions.toLabel
-import com.unciv.ui.screens.worldscreen.WorldScreen
 
 class NotificationsOverviewTable(
     viewingPlayer: Civilization,
@@ -37,13 +34,11 @@ class NotificationsOverviewTable(
     }
 
 
-    private val worldScreen = GUI.getWorldScreen()
+    private val stageWidth = overviewScreen.stage.width
 
     private val notificationLog = viewingPlayer.notificationsLog
     private val notificationTable = Table(BaseScreen.skin)
 
-    private val maxEntryWidth = worldScreen.stage.width - 20f
-
     val iconSize = 20f
 
     init {
@@ -72,9 +67,9 @@ class NotificationsOverviewTable(
                         else
                             "Current turn".toLabel()
         turnTable.add(Table().apply {
-            add(ImageGetter.getWhiteDot()).minHeight(2f).width(worldScreen.stage.width/4)
+            add(ImageGetter.getWhiteDot()).minHeight(2f).width(stageWidth / 4)
             add(turnLabel).pad(3f)
-            add(ImageGetter.getWhiteDot()).minHeight(2f).width(worldScreen.stage.width/4)
+            add(ImageGetter.getWhiteDot()).minHeight(2f).width(stageWidth / 4)
         }).row()
 
         for (category in NotificationCategory.values()){
@@ -87,18 +82,16 @@ class NotificationsOverviewTable(
             for (notification in categoryNotifications) {
                 val notificationTable = Table(BaseScreen.skin)
 
-                val labelWidth = maxEntryWidth - iconSize * notification.icons.size - 10f
-                val label = WrappableLabel(notification.text, labelWidth, Color.BLACK, 20)
+                val label = ColorMarkupLabel(notification.text, Color.BLACK, fontSize = 20)
+                    .apply { wrap = true }
 
-                notificationTable.add(label)
+                notificationTable.add(label).width(stageWidth / 2 - iconSize * notification.icons.size)
                 notificationTable.background = BaseScreen.skinStrings.getUiBackground("OverviewScreen/NotificationOverviewTable/Notification", BaseScreen.skinStrings.roundedEdgeRectangleShape)
                 notificationTable.touchable = Touchable.enabled
-                notificationTable.onClick {
-                    UncivGame.Current.resetToWorldScreen()
-                    notification.action?.execute(worldScreen)
-                }
+                if (notification.action != null)
+                    notificationTable.onClick { showOneTimeNotification(notification) }
 
-                notification.addNotificationIconsTo(notificationTable, worldScreen.gameInfo.ruleset, iconSize)
+                notification.addNotificationIconsTo(notificationTable, gameInfo.ruleset, iconSize)
 
                 turnTable.add(notificationTable).padTop(5f)
                 turnTable.padTop(20f).row()
