@@ -15,9 +15,9 @@ import com.unciv.models.translations.tr
 import com.unciv.ui.components.Fonts
 import com.unciv.ui.components.UncivTooltip.Companion.addTooltip
 import com.unciv.ui.components.extensions.disable
-import com.unciv.ui.components.extensions.keyShortcuts
-import com.unciv.ui.components.extensions.onClick
-import com.unciv.ui.components.extensions.onDoubleClick
+import com.unciv.ui.components.input.keyShortcuts
+import com.unciv.ui.components.input.onClick
+import com.unciv.ui.components.input.onDoubleClick
 import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.images.ImageGetter
 import kotlin.math.roundToInt
@@ -119,7 +119,7 @@ class ImprovementPickerScreen(
                 if (isSuperseded) shortcutKey = null
             }
 
-            var labelText = improvement.name.tr()
+            var labelText = improvement.name.tr(true)
             val turnsToBuild = if (tile.improvementInProgress == improvement.name) tile.turnsToImprovement
             else improvement.getTurnsToBuild(currentPlayerCiv, unit)
 
