@@ -9,11 +9,9 @@ import com.badlogic.gdx.math.Vector2
 import com.badlogic.gdx.scenes.scene2d.Action
 import com.badlogic.gdx.scenes.scene2d.Actor
 import com.badlogic.gdx.scenes.scene2d.Group
-import com.badlogic.gdx.scenes.scene2d.InputEvent
 import com.badlogic.gdx.scenes.scene2d.Touchable
 import com.badlogic.gdx.scenes.scene2d.actions.Actions
 import com.badlogic.gdx.scenes.scene2d.ui.Table
-import com.badlogic.gdx.scenes.scene2d.utils.ClickListener
 import com.badlogic.gdx.utils.Align
 import com.unciv.Constants
 import com.unciv.UncivGame
@@ -35,17 +33,18 @@ import com.unciv.models.helpers.MiscArrowTypes
 import com.unciv.models.ruleset.unique.LocalUniqueCache
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.ui.audio.SoundPlayer
-import com.unciv.ui.components.KeyCharAndCode
 import com.unciv.ui.components.UnitGroup
 import com.unciv.ui.components.ZoomableScrollPane
 import com.unciv.ui.components.extensions.center
 import com.unciv.ui.components.extensions.colorFromRGB
 import com.unciv.ui.components.extensions.darken
-import com.unciv.ui.components.extensions.keyShortcuts
-import com.unciv.ui.components.extensions.onActivation
-import com.unciv.ui.components.extensions.onClick
 import com.unciv.ui.components.extensions.surroundWithCircle
 import com.unciv.ui.components.extensions.toLabel
+import com.unciv.ui.components.input.KeyCharAndCode
+import com.unciv.ui.components.input.keyShortcuts
+import com.unciv.ui.components.input.onActivation
+import com.unciv.ui.components.input.onClick
+import com.unciv.ui.components.input.onRightClick
 import com.unciv.ui.components.tilegroups.TileGroup
 import com.unciv.ui.components.tilegroups.TileGroupMap
 import com.unciv.ui.components.tilegroups.TileSetStrings
@@ -53,9 +52,10 @@ import com.unciv.ui.components.tilegroups.WorldTileGroup
 import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.screens.basescreen.BaseScreen
 import com.unciv.ui.screens.basescreen.UncivStage
+import com.unciv.ui.screens.worldscreen.bottombar.BattleTableHelpers.battleAnimation
+import com.unciv.utils.Concurrency
 import com.unciv.utils.Log
-import com.unciv.utils.concurrency.Concurrency
-import com.unciv.utils.concurrency.launchOnGLThread
+import com.unciv.utils.launchOnGLThread
 import java.lang.Float.max
 
 
@@ -130,19 +130,13 @@ class WorldMapHolder(
                 continue
 
             // Right mouse click listener
-            tileGroup.addListener(object : ClickListener() {
-                init {
-                    button = Input.Buttons.RIGHT
+            tileGroup.onRightClick {
+                val unit = worldScreen.bottomUnitTable.selectedUnit
+                    ?: return@onRightClick
+                Concurrency.run("WorldScreenClick") {
+                    onTileRightClicked(unit, tileGroup.tile)
                 }
-
-                override fun clicked(event: InputEvent?, x: Float, y: Float) {
-                    val unit = worldScreen.bottomUnitTable.selectedUnit
-                        ?: return
-                    Concurrency.run("WorldScreenClick") {
-                        onTileRightClicked(unit, tileGroup.tile)
-                    }
-                }
-            })
+            }
         }
         actor = tileGroupMap
         setSize(worldScreen.stage.width, worldScreen.stage.height)
@@ -206,12 +200,10 @@ class WorldMapHolder(
     }
 
     private fun onTileRightClicked(unit: MapUnit, tile: Tile) {
-        if (UncivGame.Current.gameInfo!!.getCurrentPlayerCivilization().isSpectator()) {
-            return
-        }
         removeUnitActionOverlay()
         selectedTile = tile
         unitMovementPaths.clear()
+        if (!worldScreen.canChangeState) return
 
         // Concurrency might open up a race condition window - if worldScreen.shouldUpdate is on too
         // early, concurrent code might possibly call worldScreen.render() and then our request will be
@@ -231,6 +223,7 @@ class WorldMapHolder(
             }
             /** If we are in unit-swapping mode and didn't find a swap partner, we don't want to move or attack */
         } else {
+            // This seems inefficient as the tileToAttack is already known - but the method also calculates tileToAttackFrom
             val attackableTile = BattleHelper
                     .getAttackableEnemies(unit, unit.movement.getDistanceToTiles())
                     .firstOrNull { it.tileToAttack == tile }
@@ -239,7 +232,9 @@ class WorldMapHolder(
                 val attacker = MapUnitCombatant(unit)
                 if (!Battle.movePreparingAttack(attacker, attackableTile)) return
                 SoundPlayer.play(attacker.getAttackSound())
-                Battle.attackOrNuke(attacker, attackableTile)
+                val (damageToDefender, damageToAttacker) = Battle.attackOrNuke(attacker, attackableTile)
+                if (attackableTile.combatant != null)
+                    worldScreen.battleAnimation(attacker, damageToAttacker, attackableTile.combatant, damageToDefender)
                 localShouldUpdate = true
             } else if (unit.movement.canReach(tile)) {
                 /** ****** Right-click Move ****** */
@@ -269,13 +264,20 @@ class WorldMapHolder(
             try {
                 tileToMoveTo = selectedUnit.movement.getTileToMoveToThisTurn(targetTile)
             } catch (ex: Exception) {
-                // This is normal e.g. when selecting an air unit then right-clicking on an empty tile
-                // Or telling a ship to run onto a coastal land tile.
-                if (ex !is UnitMovement.UnreachableDestinationException)
-                    Log.error("Exception in getTileToMoveToThisTurn", ex)
+                when (ex) {
+                    is UnitMovement.UnreachableDestinationException -> {
+                        // This is normal e.g. when selecting an air unit then right-clicking on an empty tile
+                        // Or telling a ship to run onto a coastal land tile.
+                        // Do nothing
+                    }
+                    else -> {
+                        Log.error("Exception in getTileToMoveToThisTurn", ex)
+                    }
+                }
                 return@run // can't move here
             }
 
+
             worldScreen.preActionGameInfo = worldScreen.gameInfo.clone()
 
             launchOnGLThread {
@@ -508,8 +510,8 @@ class WorldMapHolder(
         actor.toFront()
 
         actor.y += actor.height
+        actor.setOrigin(Align.bottom)
         unitActionOverlays.add(actor)
-
     }
 
     /** Returns true when the civ is a human player defeated in singleplayer game */
