@@ -5,12 +5,12 @@ import com.unciv.Constants
 import com.unciv.logic.civilization.managers.ReligionState
 import com.unciv.models.ruleset.BeliefType
 import com.unciv.models.translations.tr
-import com.unciv.ui.components.KeyboardBinding
+import com.unciv.ui.components.input.KeyboardBinding
 import com.unciv.ui.components.extensions.disable
 import com.unciv.ui.components.extensions.enable
 import com.unciv.ui.components.extensions.isEnabled
-import com.unciv.ui.components.extensions.keyShortcuts
-import com.unciv.ui.components.extensions.onActivation
+import com.unciv.ui.components.input.keyShortcuts
+import com.unciv.ui.components.input.onActivation
 import com.unciv.ui.components.extensions.setSize
 import com.unciv.ui.images.IconTextButton
 import com.unciv.ui.images.ImageGetter
@@ -192,7 +192,7 @@ class NextTurnButton : IconTextButton("", null, 30) {
 class NextTurnAction(val text: String, val color: Color, val icon: String? = null, val action: () -> Unit) {
     companion object Prefabs {
         val Default = NextTurnAction("", Color.BLACK) {}
-        val Working = NextTurnAction("Working...", Color.GRAY, "NotificationIcons/Working") {}
+        val Working = NextTurnAction(Constants.working, Color.GRAY, "NotificationIcons/Working") {}
         val Waiting = NextTurnAction("Waiting for other players...",Color.GRAY, "NotificationIcons/Waiting") {}
     }
 }
